@@ -12,8 +12,18 @@ ENGINE_MAP = {
     'nhits': nhits_forecast,
 }
 
-# 순서대로 달러인덱스, 원달러환율, 미국채3개월물, 원유, 금, 은, sp500, 코스피, 니케이225, 홍콩항셍
-MI_TICKERS = ['DX-Y.NYB', 'KRW=X', '^IRX', 'CL=F', 'GC=F', 'SI=F', '^GSPC', '^KS11', '^N225', '^HSI' ]
+MIs = {
+    'DX-Y.NYB': '달러인덱스',
+    'KRW=X': '원달러환율',
+    '^IRX': '미국채3개월물',
+    'CL=F': '원유',
+    'GC=F': '금',
+    'SI=F': '은',
+    '^GSPC': 'S&P500',
+    '^KS11': '코스피',
+    '^N225': '니케이225',
+    '^HSI': '홍콩항셍',
+}
 
 def handle_cache_many_command(engine: str, tickers: list[str]):
     for ticker in tickers:
@@ -25,7 +35,7 @@ def handle_cache_many_command(engine: str, tickers: list[str]):
 
 
 def handle_cache_mi(engine: str):
-    for ticker in MI_TICKERS:
+    for ticker in MIs.keys():
         generator = ENGINE_MAP.get(engine)
         if not generator:
             raise ValueError(f"지원하지 않는 tsseer: {engine}")
@@ -94,11 +104,11 @@ def main():
             handle_cache_mi(engine)
             async def main():
                 if engine == 'nbeats' or engine == 'nhits':
-                    await filter_mydarts_by_trend_and_anomaly(MI_TICKERS, "상승", refresh=True)
-                    await filter_mydarts_by_trend_and_anomaly(MI_TICKERS, "하락", refresh=True)
+                    await filter_mydarts_by_trend_and_anomaly(MIs.keys(), "상승", refresh=True)
+                    await filter_mydarts_by_trend_and_anomaly(MIs.keys(), "하락", refresh=True)
                 elif engine == 'prophet':
-                    await filter_prophet_by_trend_and_anomaly(MI_TICKERS, "상승", refresh=True)
-                    await filter_prophet_by_trend_and_anomaly(MI_TICKERS, "하락", refresh=True)
+                    await filter_prophet_by_trend_and_anomaly(MIs.keys(), "상승", refresh=True)
+                    await filter_prophet_by_trend_and_anomaly(MIs.keys(), "하락", refresh=True)
             asyncio.run(main())
         else:
             print("mi 캐시는 'all' 만 허용 됩니다.")
\ No newline at end of file
