@@ -1,3 +1,7 @@
+from __future__ import annotations
+
+from uuid import uuid4
+from darts.utils.likelihood_models import GaussianLikelihood
 from typing import Any
 
 import pandas as pd
@@ -68,8 +72,168 @@ def enough_length(ts: TimeSeries, in_len: int, out_len: int) -> bool:
     return len(ts) >= (in_len + out_len)
 
 
-def train_and_forecast(series_scaler_dict: dict[str, TimeSeries | Scaler],
-                       train_val_dict: dict[str, TimeSeries | Scaler]) -> dict[str, Any] | None:
+def train_and_forecast(series_scaler_dict: dict[str, Any],
+                       train_val_dict: dict[str, Any]) -> dict[str, Any] | None:
+    # 두 단계 학습(탐색→재학습)을 없애고, 한 번의 학습에서 체크포인트로 ‘val_loss 최저’ 가중치를 저장→복원해 바로 예측에 쓰도록 고친 코드
+    INPUT_LEN  = 60   # 과거 60일
+    OUTPUT_LEN = 15   # 미래 15영업일
+    MIN_LEN    = INPUT_LEN + OUTPUT_LEN  # 75
+    PRED_STEPS = 60   # 예측 60영업일
+
+    # ---- 분리/스케일된 입력 꺼내기 ----
+    target_train = train_val_dict['target_train']
+    target_val   = train_val_dict['target_val']
+
+    # 전체(스케일된) 학습 대상 — 이번 버전은 1회 학습에서 바로 전체를 사용
+    target_scaled = series_scaler_dict['target_scaled']
+    volume_scaled = series_scaler_dict['volume_scaled']
+
+    # 역스케일러
+    target_scaler = series_scaler_dict['target_scaler']
+
+    # ---- 길이 검사 (train/val 모두 충분해야 함) ----
+    if not all(
+        enough_length(ts, INPUT_LEN, OUTPUT_LEN)
+        for ts in (target_train, target_val)
+    ):
+        mylogger.warning(
+            f"skip training: series shorter than {MIN_LEN} steps "
+            f"(train={len(target_train)}, val={len(target_val)})"
+        )
+        return None
+
+    # ---- covariates 정렬/확장 (학습: 전체, 검증: val 구간) ----
+    # 학습용 past covariate: 전체 범위를 target_scaled 기준으로 뒤로 INPUT_LEN만큼 확장
+    volume_scaled_backward_extended_full = extend_covariate_backwards(
+        INPUT_LEN, target_scaled, volume_scaled
+    )
+    # 검증용 past covariate: val 구간에 맞춰 동일 스케일로 확장
+    volume_scaled_backward_extended_val = extend_covariate_backwards(
+        INPUT_LEN, target_val, volume_scaled
+    )
+
+    if (volume_scaled_backward_extended_full is None or
+        volume_scaled_backward_extended_val  is None):
+        raise RuntimeError("covariate extension failed (None returned)")
+
+    # ----(선택) 범위/주기 점검 로그----
+    def covariate_ok(past, target, in_len, out_len):
+        need_start = target.start_time() - in_len * target.freq
+        need_end   = target.end_time()
+        mylogger.debug(f"cov.start ⩽ {need_start} ? {past.start_time() <= need_start}")
+        mylogger.debug(f"cov.end   ≥ {need_end} ? {past.end_time() >= need_end}")
+        mylogger.debug(f"freq 동일 ? {past.freq == target.freq}")
+
+    covariate_ok(volume_scaled_backward_extended_full, target_scaled, in_len=INPUT_LEN-1, out_len=OUTPUT_LEN)
+    covariate_ok(volume_scaled_backward_extended_val,  target_val,    in_len=INPUT_LEN-1, out_len=OUTPUT_LEN)
+
+    # ---- NaN 검사 ----
+    def validate_no_nan_in_series():
+        for name, ts in zip(
+            ["target_train", "target_val", "target_scaled",
+             "volume_scaled_backward_extended_full", "volume_scaled_backward_extended_val"],
+            [target_train, target_val, target_scaled,
+             volume_scaled_backward_extended_full,   volume_scaled_backward_extended_val],
+        ):
+            df = ts.to_dataframe()
+            nan_mask = df.isna().any(axis=1)
+            if nan_mask.any():
+                nan_dates = ts.time_index[nan_mask]
+                first_five = list(nan_dates[:5])
+                raise ValueError(
+                    f"{name} contains NaN values "
+                    f"(example dates: {first_five} … total {nan_mask.sum()} rows)."
+                )
+        mylogger.debug("All series are NaN-free")
+
+    validate_no_nan_in_series()
+
+    # ──────────────────────────────────────────────────────────
+    # NBEATS 모델 (1회 학습, 체크포인트 저장)
+    # ──────────────────────────────────────────────────────────
+    early_stop = EarlyStopping(monitor="val_loss", patience=int(INPUT_LEN * 0.25), mode="min")
+
+    # 체크포인트 저장/복원을 위해 model_name 부여
+    model_name = f"nbeats_{uuid4().hex[:8]}"
+
+    model = NBEATSModel(
+        input_chunk_length = INPUT_LEN,
+        output_chunk_length= OUTPUT_LEN,
+        generic_architecture=True,
+        n_epochs=1000,
+        optimizer_kwargs={"lr": 1e-3},
+        likelihood=GaussianLikelihood(),
+        batch_size=64,
+        random_state=42,
+
+        # ✅ 체크포인트 저장 설정
+        model_name=model_name,
+        save_checkpoints=True,  # 내부적으로 val_loss 기준 체크포인트를 저장
+
+        pl_trainer_kwargs={
+            "accelerator": "gpu" if torch.cuda.is_available() else "cpu",
+            "gradient_clip_val": 1.0,
+            "callbacks": [early_stop],  # ModelCheckpoint는 Darts가 내부에서 생성
+        },
+    )
+
+    # ──────────────────────────────────────────────────────────
+    # 학습 (한 번만): series=전체 스케일 데이터, 검증은 val로
+    # ──────────────────────────────────────────────────────────
+    model.fit(
+        series                 = target_scaled,  # 전체
+        past_covariates        = volume_scaled_backward_extended_full,
+        val_series             = target_val,
+        val_past_covariates    = volume_scaled_backward_extended_val,
+        verbose=True,
+    )
+
+    # ──────────────────────────────────────────────────────────
+    # ✅ 최적(베스트) 가중치 복원
+    # ──────────────────────────────────────────────────────────
+    # Darts가 저장한 ‘val_loss 최저’ 체크포인트에서 동일 아키텍처 모델을 로드
+    best_model = NBEATSModel.load_from_checkpoint(
+        model_name=model_name,
+        best=True,          # 가장 좋은 체크포인트 선택
+        # work_dir=None,    # 별도 작업 디렉터리를 쓰면 지정
+    )
+
+    # ──────────────────────────────────────────────────────────
+    # 예측
+    # ──────────────────────────────────────────────────────────
+    # 과거 공변량이 필요할 수 있으므로(반복 예측) 미래 구간까지 길이 보강
+    past_cov_for_pred = extend_future_covariates(
+        PRED_STEPS - OUTPUT_LEN,  # OUTPUT_LEN씩 굴리는 반복 예측 대비 여유분
+        volume_scaled_backward_extended_full
+    )
+
+    forecast = best_model.predict(
+        n               = PRED_STEPS,
+        past_covariates = past_cov_for_pred,
+        num_samples     = 80,
+        show_warnings   = False,
+    )
+
+    # ──────────────────────────────────────────────────────────
+    # 스케일 복원 & 신뢰구간/평균 산출
+    # ──────────────────────────────────────────────────────────
+    target_inv   = target_scaler.inverse_transform(target_scaled)
+    forecast_inv = target_scaler.inverse_transform(forecast)
+
+    lower_ts = forecast_inv.quantile(0.10)
+    upper_ts = forecast_inv.quantile(0.90)
+    mean_ts  = forecast_inv.mean()
+
+    return {
+        "fcst_mean_ts": mean_ts,
+        "actual_ts":    target_inv,
+        "lower_ts":     lower_ts,
+        "upper_ts":     upper_ts,
+    }
+
+
+def train_and_forecast_double_study(series_scaler_dict: dict[str, TimeSeries | Scaler],
+                                    train_val_dict: dict[str, TimeSeries | Scaler]) -> dict[str, Any] | None:
     INPUT_LEN = 60  # 과거 60일을 보고
     OUTPUT_LEN = 15  # 미래 15영업일 예측
     MIN_LEN = INPUT_LEN + OUTPUT_LEN  # 75
@@ -236,7 +400,7 @@ def train_and_forecast(series_scaler_dict: dict[str, TimeSeries | Scaler],
     )
 
     # ──────────────────────────────────────────────────────────
-    # 예측 (미래 60영업일)
+    # 예측 (미래 n영업일)
     # ──────────────────────────────────────────────────────────
     PRED_STEPS = 60
     forecast = final_model.predict(
