@@ -146,6 +146,55 @@ def prophet_forecast(ticker: str, *, refresh: bool = False, cache_only: bool = F
 @redis_async_cached(prefix=REDIS_PREFIX_MAP['trend'], key_factory=trend_key_factory)
 async def filter_prophet_by_trend_and_anomaly(tickers: list[str], trend: Literal["상승", "하락"], *,
                                               refresh: bool = False, cache_only: bool = False) -> list[dict]:
+    """
+    주어진 종목 목록(`tickers`)에 대해 Redis에 저장된 Prophet 예측 데이터를 기반으로
+    지정된 추세(`trend`)와 이상치 점수(`anomaly_score`) 조건을 만족하는 항목만 필터링하여 반환합니다.
+
+    ### 주요 동작
+    1. Redis에서 Prophet 예측 데이터 일괄 조회 (`mget`).
+    2. 각 종목별로 다음 조건을 만족하는 경우 결과에 포함:
+       - 예측된 추세(`payload['trend']`)가 입력한 `trend`와 동일해야 함.
+       - 이상치 점수(`anomaly_score`)가 존재해야 함.
+       - **상승(`trend="상승"`)**: 이상치 점수가 `-0.01` 이하인 경우만 포함.
+       - **하락(`trend="하락"`)**: 이상치 점수가 `0.01` 이상인 경우만 포함.
+    3. 최종 결과를 이상치 점수의 절대값 기준 내림차순으로 정렬 후 반환.
+
+    ### 매개변수
+    - `tickers (list[str])`:
+        필터링할 종목 코드 목록.
+    - `trend (Literal["상승", "하락"])`:
+        필터링할 목표 추세.
+    - `refresh (bool, optional)`:
+        캐시 무효화 여부. 기본값은 `False`.
+    - `cache_only (bool, optional)`:
+        캐시에서만 조회하고 미존재 시 생략 여부. 기본값은 `False`.
+
+    ### 반환값
+    - `list[dict]`: 다음 필드를 포함하는 딕셔너리 목록
+        ```python
+        [
+            {
+                "ticker": str,          # 종목 코드
+                "trend": str,           # 예측 추세 ("상승" 또는 "하락")
+                "anomaly_score": float  # 이상치 점수
+            },
+            ...
+        ]
+        ```
+
+    ### 예시
+    ```python
+    results = await filter_prophet_by_trend_and_anomaly(
+        tickers=["005930", "000660", "035420"],
+        trend="상승"
+    )
+    # 결과 예시:
+    # [
+    #   {"ticker": "005930", "trend": "상승", "anomaly_score": -0.05},
+    #   {"ticker": "000660", "trend": "상승", "anomaly_score": -0.02},
+    # ]
+    ```
+    """
     r = get_redis_client_async()
     # prophet을 사용해서 데이터를 만들었을 경우
     keys = [f"{REDIS_PREFIX_MAP['prophet']}:{str(t).strip()}" for t in tickers]
